package com.xisc.pm.app.service;

import com.xisc.pm.api.dto.PmMaterialInfoHangOrderDTO;
import com.xisc.pm.domain.entity.MmSlab;
import com.xisc.pm.domain.vo.MmPlateVO;
import com.xisc.pm.domain.vo.QdToPoOrderVO1;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 钢板主数据表(MmPlate)应用服务
 *
 * @author 蒋雨航 yuhang.jiang02@hand-china.com
 * @since 2024-08-14 15:37:09
 */
public interface PmFinishedProductActAsService {

    /**
     * 余材钢板充当材料信息列表
     *
     * @param mmPlatesVO 查询条件
     * @return 返回值
     */
    List<MmPlateVO> surplusSteelPlateList(MmPlateVO mmPlatesVO, Long organizationId);

    /**
     * 余材钢板目标合同查询
     *
     * @param qdToPoOrderVO1
     * @param organizationId
     * @return
     */
    List<QdToPoOrderVO1> surplusSteelPlateOrderList(QdToPoOrderVO1 qdToPoOrderVO1, Long organizationId);

    /**
     * 余材钢板充当挂合同
     *
     * @param dto
     * @param organizationId
     */
    String surplusSteelPlateHangOrder(PmMaterialInfoHangOrderDTO dto, Long organizationId);

    /**
     * 余材板坯充当材料信息列表
     *
     * @param mmSlab
     * @param organizationId
     * @return
     */
    List<MmSlab> surplusSlabList(MmSlab mmSlab, Long organizationId);

    /**
     * 余材板坯充当目标合同查询
     *
     * @param qdToPoOrderVO1
     * @param organizationId
     * @return
     */
    List<QdToPoOrderVO1> surplusSlabOrderList(QdToPoOrderVO1 qdToPoOrderVO1, Long organizationId);

    /**
     * 余材板坯充当挂合同
     *
     * @param dto
     * @param organizationId
     */
    void surplusSlabHangOrder(PmMaterialInfoHangOrderDTO dto, Long organizationId);

    /**
     * 余材钢板充当脱合同
     *
     * @param dto
     * @param organizationId
     */
    void surplusSteelPlateBreakOrder(PmMaterialInfoHangOrderDTO dto, Long organizationId);

    /**
     * 余材板坯充当脱合同
     *
     * @param dto
     * @param organizationId
     */
    void surplusSlabBreakOrder(PmMaterialInfoHangOrderDTO dto, Long organizationId);

    /**
     * 成品充当统计查询
     *
     * @param timeTo
     * @param timeFrom
     * @param orderTypeCode
     * @param organizationId
     * @return
     */
    Map<String, Object> statisticsQuery(Date timeTo, Date timeFrom, Integer orderTypeCode, Long organizationId, int page, int size, Long createdBy);
}

