package com.xisc.pm.app.service.imports;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xisc.pm.domain.entity.PmPdTmpYieldBt;
import com.xisc.pm.domain.entity.PmPdTmpYieldBtTra;
import com.xisc.pm.domain.repository.PmPdTmpYieldBtRepository;
import com.xisc.pm.domain.repository.PmPdTmpYieldBtTraRepository;
import io.choerodon.core.exception.CommonException;
import io.choerodon.core.oauth.DetailsHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.hzero.boot.imported.app.service.IBatchImportService;
import org.hzero.boot.imported.infra.validator.annotation.ImportService;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @since 2024/7/26 9:36
 */
@Slf4j
@ImportService(templateCode = "PM.PD_TMP_YIELD_BT")
public class PmPdTmpYieldBtImportImpl implements IBatchImportService {

    private final ObjectMapper objectMapper;

    private final PmPdTmpYieldBtRepository pmPdTmpYieldBtRepository;

    private final PmPdTmpYieldBtTraRepository pmPdTmpYieldBtTraRepository;

    public PmPdTmpYieldBtImportImpl(ObjectMapper objectMapper, PmPdTmpYieldBtRepository pmPdTmpYieldBtRepository, PmPdTmpYieldBtTraRepository pmPdTmpYieldBtTraRepository) {
        this.objectMapper = objectMapper;
        this.pmPdTmpYieldBtRepository = pmPdTmpYieldBtRepository;
        this.pmPdTmpYieldBtTraRepository = pmPdTmpYieldBtTraRepository;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean doImport(List<String> data) {
        // 如果集合为空，则返回
        if (CollectionUtils.isEmpty(data)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 获取租户id和当前时间以及用户信息
        Long tenantId = 0L;
        Date now = new Date();
        String username = DetailsHelper.getUserDetails().getUsername();
        if (DetailsHelper.getUserDetails() != null && DetailsHelper.getUserDetails().getTenantId() != null) {
            tenantId = DetailsHelper.getUserDetails().getTenantId();
        }
        // 定义要插入的List
        List<PmPdTmpYieldBt> list = new ArrayList<>(data.size());
        List<PmPdTmpYieldBtTra> tralist = new ArrayList<>(data.size());

        // 数据转对象
        for (String json : data) {
            try {
                PmPdTmpYieldBt pmPdTmpYieldBt = objectMapper.readValue(json, PmPdTmpYieldBt.class);
                pmPdTmpYieldBt.setTenantId(tenantId);
                list.add(pmPdTmpYieldBt);

            } catch (JsonProcessingException e) {
                log.error(e.getMessage(), e);
            }
        }

        // 如果集合为空，则结束
        if (CollectionUtils.isEmpty(list)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        // 插入表
        pmPdTmpYieldBtRepository.batchInsertSelective(list);

        // 获取履历表数据
        for (PmPdTmpYieldBt pmPdTmpYieldBt : list) {
            PmPdTmpYieldBtTra pmPdTmpYieldBtTra = new PmPdTmpYieldBtTra();
            BeanUtils.copyProperties(pmPdTmpYieldBt, pmPdTmpYieldBtTra);
            // 设置操作类型为新增（I），并记录操作时间和用户
            pmPdTmpYieldBtTra.setOperationType("I");
            pmPdTmpYieldBtTra.setOperationTime(now);
            pmPdTmpYieldBtTra.setOperationUser(username);
            tralist.add(pmPdTmpYieldBtTra);
        }
        // 插入履历表
        pmPdTmpYieldBtTraRepository.batchInsertSelective(tralist);
        return true;
    }
}
