package com.xisc.pm.app.service.strategy.pdrule;

import java.util.Map;

import org.springframework.stereotype.Component;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmContMscTech;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.PdRuleItemType;

import groovyjarjarantlr4.v4.runtime.misc.Utils;
import io.choerodon.core.exception.CommonException;

/**
 * 缓冷代码及探伤时期代码一致（C03）
 * 
 * @author jun.zeng
 * 
 *
 * 
 */
@Component
public class CoolAndUltStgRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.COOL_AND_ULT_STG, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);

        PmContMscTech axisPmContMscTech = (PmContMscTech) bizData.get(RuleSetConstants.AXIS_PM_CONT_MSC_TECH);
        PmContMscTech pmContMscTech = (PmContMscTech) bizData.get(RuleSetConstants.PM_CONT_MSC_TECH);

        if (axisPmContMscTech == null || pmContMscTech == null) {
            // 读生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        if(Utils.equals(axisPmContMscTech.getCoolCode(), pmContMscTech.getCoolCode())
                && Utils.equals(axisPmContMscTech.getUltStgCode(), pmContMscTech.getUltStgCode())) {
            return true ;
        }
        // 缓冷代码或超声探伤时期代码与轴合同不一致。
        throw new CommonException("pm.pm_pd_rule_set_011") ;
    
    }

}
