package com.xisc.pm.app.service.strategy.pdrule;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmProContProdTtr;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.PdRuleItemType;
import io.choerodon.core.exception.CommonException;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * 边部形态相等 （C03）
 * 
 * @author jun.zeng
 * 
 *
 * 
 */
@Component
public class ProdSideMarkRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.PROD_SIDE_MARK, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);

        PmProContProdTtr axisPmProContProdTtr = (PmProContProdTtr) bizData
                .get(RuleSetConstants.AXIS_PM_PRO_CONT_PROD_ATTR);
        PmProContProdTtr pmProContProdTtr = (PmProContProdTtr) bizData.get(RuleSetConstants.PM_PRO_CONT_PROD_ATTR);

        if (axisPmProContProdTtr == null || pmProContProdTtr == null) {
            // 读生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        if (axisPmProContProdTtr.getProdSideMark() != null
                && axisPmProContProdTtr.getProdSideMark().equals(pmProContProdTtr.getProdSideMark())) {
            return true;
        }
        throw new CommonException("pm.pm_pd_rule_set_009", axisPmProContProdTtr.getOrderNo(), pmProContProdTtr.getOrderNo());
    }

}
