package com.xisc.pm.app.service.strategy.pdrule;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmProContProdTtr;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.FormPlateMode;
import com.xisc.pm.infra.enums.PdRuleItemType;

import io.choerodon.core.exception.CommonException;

/**
 * 
 * @author jun.zeng
 *
 */
@Component
public class SampleModeCodeBRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.SAMPLE_MODE_CODE_B, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);
        String formPlateMode = (String) bizData.get(RuleSetConstants.FORM_PLATE_MODE);

        PmProContProdTtr axisPmProContProdTtr = (PmProContProdTtr) bizData.get(RuleSetConstants.AXIS_PM_PRO_CONT_PROD_ATTR);
        PmProContProdTtr pmProContProdTtr = (PmProContProdTtr) bizData.get(RuleSetConstants.PM_PRO_CONT_PROD_ATTR);
        if (null == axisPmProContProdTtr || StringUtils.isBlank(axisPmProContProdTtr.getSampleModeCode())) {
            // 读生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        
        if(null != pmProContProdTtr) {
            if("B".equals(axisPmProContProdTtr.getSampleModeCode())) {
                if(!FormPlateMode.A1.getCode().equals(formPlateMode)){
                    // 自然张合同只能A1型组板
                    throw new CommonException("pm.pm_pd_rule_set_046" , axisPmProContProdTtr.getOrderNo());
                }
            } 
        }
        return true ;
    }

}
