package com.xisc.pm.app.service.strategy.pdrule;

import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import com.xisc.pm.api.dto.RuleParamDTO;
import com.xisc.pm.app.service.strategy.RuleExecStrategyFactory;
import com.xisc.pm.domain.entity.PmProContProdTtr;
import com.xisc.pm.infra.constant.RuleSetConstants;
import com.xisc.pm.infra.enums.PdRuleItemType;

import io.choerodon.core.exception.CommonException;

/**
 * 四毛交货合同单独组板（C03）
 * 
 * @author jun.zeng
 * 
 */
@Component
public class TrimCodeMatchRuleExecStrategy implements RuleExecStrategy {

    @Override
    public void afterPropertiesSet() throws Exception {
        RuleExecStrategyFactory.register(PdRuleItemType.TRIM_CODE_MATCH, this);
    }

    @Override
    public boolean execRule(RuleParamDTO ruleParam) {
        Map<String, Object> bizData = ruleParam.getBizData();
        String orderNo = (String) bizData.get(RuleSetConstants.ORDER_NO);
        String mscLineNo = (String) bizData.get(RuleSetConstants.MSC_LINE_NO);

        PmProContProdTtr axisPmProContProdTtr = (PmProContProdTtr) bizData.get(RuleSetConstants.AXIS_PM_PRO_CONT_PROD_ATTR);
        PmProContProdTtr pmProContProdTtr = (PmProContProdTtr) bizData.get(RuleSetConstants.PM_PRO_CONT_PROD_ATTR);

        if (axisPmProContProdTtr == null || pmProContProdTtr == null) {
            // 读生产合同池没有找到记录，合同号【order_no】，产线号【msc_line_no】
            throw new CommonException("pm.pm_pd_rule_set_001", orderNo, mscLineNo);
        }
        if(StringUtils.isNotBlank(axisPmProContProdTtr.getTrimCode()) 
                && "0".equals(axisPmProContProdTtr.getTrimCode()) ) {
            // 钢板缓冷代码与轴合同不匹配。
            throw new CommonException("pm.pm_pd_rule_set_025") ;
        }
        if(StringUtils.isNotBlank(pmProContProdTtr.getTrimCode()) 
                && "0".equals(pmProContProdTtr.getTrimCode()) ) {
            // 钢板缓冷代码与轴合同不匹配。
            throw new CommonException("pm.pm_pd_rule_set_026") ;
        }
        return true;
    }

}
