package com.xisc.pm.app.service.validator;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.xisc.pm.app.service.PmIngotThickAddvBtService;
import com.xisc.pm.domain.entity.PmIngotThickAddvBt;
import com.xisc.pm.domain.repository.PmIngotThickAddvBtRepository;
import com.xisc.pm.infra.util.Utils;
import io.choerodon.core.exception.CommonException;
import io.choerodon.core.oauth.DetailsHelper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.hzero.boot.imported.app.service.BatchValidatorHandler;
import org.hzero.boot.imported.infra.validator.annotation.ImportValidator;
import org.hzero.boot.imported.infra.validator.annotation.ImportValidators;
import org.hzero.core.message.MessageAccessor;
import org.hzero.mybatis.domian.Condition;
import org.hzero.mybatis.util.Sqls;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @date 2024/12/12 14:08
 * @description
 */
@Slf4j
@ImportValidators({@ImportValidator(templateCode = "PM.INGOT_THICK_ADDV_BT")})
public class PmIngotThickAddvBtValidator extends BatchValidatorHandler {

    @Autowired
    private ObjectMapper objectMapper;

    @Autowired
    private PmIngotThickAddvBtRepository pmIngotThickAddvBtRepository;

    @Autowired
    private PmIngotThickAddvBtService pmIngotThickAddvBtService;

    @Override
    public boolean validate(List<String> data) {
        if (CollectionUtils.isEmpty(data)) {
            throw new CommonException("pm.import_data_is_empty_list");
        }
        boolean flag = true;
        Long tenantId = 0L;
        if (DetailsHelper.getUserDetails() != null && DetailsHelper.getUserDetails().getTenantId() != null) {
            tenantId = DetailsHelper.getUserDetails().getTenantId();
        }
        List<PmIngotThickAddvBt> list = new ArrayList<>(data.size());
        Map<PmIngotThickAddvBt, Integer> idxMap = new HashMap<>(data.size());

        for (int i = 0; i < data.size(); i++) {
            try {
                // 转换对象
                PmIngotThickAddvBt pmIngotThickAddvBt = objectMapper.readValue(data.get(i), PmIngotThickAddvBt.class);
                pmIngotThickAddvBt.setTenantId(tenantId);

                Integer integer = idxMap.putIfAbsent(pmIngotThickAddvBt, i);
                if (integer != null && integer != i) {
                    // 重复数据
                    addErrorMsg(i, MessageAccessor.getMessage("pm.exist_same_data").getDesc());
                    continue;
                }
                list.add(pmIngotThickAddvBt);
            } catch (JsonProcessingException e) {
                log.error(e.getMessage(), e);
            }
        }

        // 如果集合为空，则结束
        if (CollectionUtils.isEmpty(list)) {
            return flag;
        }

        // 新增记录之间校验区间是否交叉
        Map<String, List<PmIngotThickAddvBt>> pmIngotThickAddvBtMap = list.stream()
                .collect(Collectors.groupingBy(pmIngotThickAddvBtService::getAddvKey));
        for (Map.Entry<String, List<PmIngotThickAddvBt>> entry : pmIngotThickAddvBtMap.entrySet()) {
            List<PmIngotThickAddvBt> thickAddvBtList = entry.getValue();
            // 新增记录之间校验区间是否交叉
            for (int i = 0; i < thickAddvBtList.size() - 1; i++) {
                for (int j = i + 1; j < thickAddvBtList.size(); j++) {
                    // 校验钢板厚度区间是否交叉
                    boolean plateThickIntervalCheck = Utils.isIntersect(thickAddvBtList.get(i).getPlateThickFrom(),
                            thickAddvBtList.get(i).getPlateThickTo(), thickAddvBtList.get(j).getPlateThickFrom(),
                            thickAddvBtList.get(j).getPlateThickTo());
                    if (!plateThickIntervalCheck) {
                        continue;
                    }

                    // 组合板宽度
                    boolean combinePlateWidthIntervalCheck = Utils.isIntersect(thickAddvBtList.get(i).getCombinePlateWidthFrom(),
                            thickAddvBtList.get(i).getCombinePlateWidthTo(), thickAddvBtList.get(j).getCombinePlateWidthFrom(),
                            thickAddvBtList.get(j).getCombinePlateWidthTo());
                    if (!combinePlateWidthIntervalCheck) {
                        continue;
                    }
                    addErrorMsg(i, MessageAccessor.getMessage("pm.thick_addv_bt_003",
                            new Object[]{thickAddvBtList.get(j).getPlateThickFrom(),
                                    thickAddvBtList.get(j).getPlateThickTo(), thickAddvBtList.get(j).getCombinePlateWidthFrom(),
                                    thickAddvBtList.get(j).getCombinePlateWidthTo()}).getDesc());
                    flag = false;
                }
            }
        }
        // 与数据库记录校验区间是否交叉
        // 与数据库记录校验区间是否交叉
        List<String> ingotTypeList = list.stream().map(PmIngotThickAddvBt::getIngotType).distinct()
                .collect(Collectors.toList());
        List<String> heatTreatModeList = list.stream().map(PmIngotThickAddvBt::getHeatTreatMode).distinct()
                .collect(Collectors.toList());
        List<String> rollModeList = list.stream().map(PmIngotThickAddvBt::getRollMode).distinct()
                .collect(Collectors.toList());
        List<String> rollDirectCodeList = list.stream().map(PmIngotThickAddvBt::getRollDirectCode).distinct()
                .collect(Collectors.toList());
        List<Long> idList = list.stream().filter(item -> item.getPitabId() != null).map(PmIngotThickAddvBt::getPitabId).collect(Collectors.toList());
        List<PmIngotThickAddvBt> exisPmIgnotThickAddvBtList = pmIngotThickAddvBtRepository.selectByCondition(Condition.builder(PmIngotThickAddvBt.class)
                .andWhere(Sqls.custom().andEqualTo(PmIngotThickAddvBt.FIELD_TENANT_ID, tenantId))
                .andWhere(Sqls.custom().andIn(PmIngotThickAddvBt.FIELD_INGOT_TYPE, ingotTypeList))
                .andWhere(Sqls.custom().andIn(PmIngotThickAddvBt.FIELD_HEAT_TREAT_MODE, heatTreatModeList))
                .andWhere(Sqls.custom().andIn(PmIngotThickAddvBt.FIELD_ROLL_MODE, rollModeList))
                .andWhere(Sqls.custom().andIn(PmIngotThickAddvBt.FIELD_ROLL_DIRECT_CODE, rollDirectCodeList))
                .andWhere(Sqls.custom().andNotIn(PmIngotThickAddvBt.FIELD_PITAB_ID, idList, true))
                .build());
        if (org.apache.commons.collections.CollectionUtils.isNotEmpty(exisPmIgnotThickAddvBtList)) {
            Map<String, List<PmIngotThickAddvBt>> existAddvMap = exisPmIgnotThickAddvBtList.stream().
                    collect(Collectors.groupingBy(pmIngotThickAddvBtService::getAddvKey));
            for (int i = 0; i < list.size(); i++) {
                if (existAddvMap.containsKey(pmIngotThickAddvBtService.getAddvKey(list.get(i)))) {
                    List<PmIngotThickAddvBt> thickAddvBtList = existAddvMap.get(pmIngotThickAddvBtService.getAddvKey(list.get(i)));
                    for (PmIngotThickAddvBt ingotThickAddvBt : thickAddvBtList) {
                        // 校验钢板厚度区间是否交叉
                        boolean rollThickIntervalCheck = Utils.isIntersect(list.get(i).getPlateThickFrom(),
                                list.get(i).getPlateThickTo(), ingotThickAddvBt.getPlateThickFrom(),
                                ingotThickAddvBt.getPlateThickTo());
                        if (!rollThickIntervalCheck) {
                            continue;
                        }

                        // 校验组合板宽度区间是否交叉
                        boolean combinePlateWidthIntervalCheck = Utils.isIntersect(list.get(i).getCombinePlateWidthFrom(),
                                list.get(i).getCombinePlateWidthTo(), ingotThickAddvBt.getCombinePlateWidthFrom(),
                                ingotThickAddvBt.getCombinePlateWidthTo());
                        if (!combinePlateWidthIntervalCheck) {
                            continue;
                        }

                        addErrorMsg(i, MessageAccessor.getMessage("pm.thick_addv_bt_004",
                                new Object[]{ingotThickAddvBt.getPlateThickFrom(),
                                        ingotThickAddvBt.getPlateThickTo(), ingotThickAddvBt.getCombinePlateWidthFrom(),
                                        ingotThickAddvBt.getCombinePlateWidthTo()}).getDesc());
                        flag = false;
                    }
                }
            }
        }
        return flag;
    }
}
