package com.xisc.pm.domain.entity;

import com.fasterxml.jackson.annotation.JsonInclude;
import io.choerodon.mybatis.annotation.ModifyAudit;
import io.choerodon.mybatis.annotation.VersionAudit;
import io.choerodon.mybatis.domain.AuditDomain;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Objects;


/**
 * 钢锭轧制规格及切边量基准表(PmIgnotSpecTrimBt)实体类
 *
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @since 2024-07-24 15:01:54
 */

@Getter
@Setter
@ApiModel("钢锭轧制规格及切边量基准表")
@VersionAudit
@ModifyAudit
@JsonInclude(value = JsonInclude.Include.NON_NULL)
@Table(name = "PM_INGOT_SPEC_TRIM_BT")
public class PmIngotSpecTrimBt extends AuditDomain implements Serializable {
    private static final long serialVersionUID = 718619249159615814L;

    public static final String FIELD_PISTB_ID = "pistbId";
    public static final String FIELD_INGOT_TYPE = "ingotType";
    public static final String FIELD_GUARANTEE_QUALITY = "guaranteeQuality";
    public static final String FIELD_MIN_ROLL_THICK = "minRollThick";
    public static final String FIELD_MAX_ROLL_THICK = "maxRollThick";
    public static final String FIELD_MIN_ROLL_WIDTH = "minRollWidth";
    public static final String FIELD_MAX_ROLL_WIDTH = "maxRollWidth";
    public static final String FIELD_ROLL_LEN_FROM = "rollLenFrom";
    public static final String FIELD_ROLL_LEN_TO = "rollLenTo";
    public static final String FIELD_ROLL_LINE_NO = "rollLineNo";
    public static final String FIELD_WITH_SIDE_FLAG = "withSideFlag";
    public static final String FIELD_ROLL_DIRECT_CODE = "rollDirectCode";
    public static final String FIELD_TRIM_WIDTH = "trimWidth";
    public static final String FIELD_TENANT_ID = "tenantId";
    public static final String FIELD_ATTRIBUTE1 = "attribute1";
    public static final String FIELD_ATTRIBUTE2 = "attribute2";
    public static final String FIELD_ATTRIBUTE3 = "attribute3";
    public static final String FIELD_ATTRIBUTE4 = "attribute4";
    public static final String FIELD_ATTRIBUTE5 = "attribute5";

    @ApiModelProperty("主键ID")
    @Id
    @GeneratedValue
    private Long pistbId;

    @ApiModelProperty(value = "锭坯型", required = true)
    @NotBlank
    private String ingotType;

    @ApiModelProperty(value = "是否保性能", required = true)
    @NotBlank
    private String guaranteeQuality;

    @ApiModelProperty(value = "轧制厚度起")
    private BigDecimal minRollThick;

    @ApiModelProperty(value = "轧制厚度止")
    private BigDecimal maxRollThick;

    @ApiModelProperty(value = "轧制宽度起")
    private BigDecimal minRollWidth;

    @ApiModelProperty(value = "轧制宽度止")
    private BigDecimal maxRollWidth;

    @ApiModelProperty(value = "轧制长度起")
    private BigDecimal rollLenFrom;

    @ApiModelProperty(value = "轧制长度止")
    private BigDecimal rollLenTo;

    @ApiModelProperty(value = "轧制产线号", required = true)
    @NotBlank
    private String rollLineNo;

    @ApiModelProperty(value = "切边标记", required = true)
    @NotBlank
    private String withSideFlag;

    @ApiModelProperty(value = "轧制方向代码", required = true)
    @NotBlank
    private String rollDirectCode;

    @ApiModelProperty(value = "切边量", required = true)
    @NotNull
    private BigDecimal trimWidth;

    @ApiModelProperty(value = "租户ID", required = true)
    @NotNull
    private Long tenantId;

    private String attribute1;

    private String attribute2;

    private String attribute3;

    private String attribute4;

    private String attribute5;

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        PmIngotSpecTrimBt that = (PmIngotSpecTrimBt) o;
        return Objects.equals(pistbId, that.pistbId) && Objects.equals(ingotType, that.ingotType) &&
                Objects.equals(guaranteeQuality, that.guaranteeQuality) && Objects.equals(minRollThick, that.minRollThick) &&
                Objects.equals(maxRollThick, that.maxRollThick) && Objects.equals(minRollWidth, that.minRollWidth) &&
                Objects.equals(maxRollWidth, that.maxRollWidth) && Objects.equals(rollLenFrom, that.rollLenFrom) &&
                Objects.equals(rollLenTo, that.rollLenTo) && Objects.equals(rollLineNo, that.rollLineNo) &&
                Objects.equals(withSideFlag, that.withSideFlag) && Objects.equals(rollDirectCode, that.rollDirectCode) &&
                Objects.equals(trimWidth, that.trimWidth) && Objects.equals(tenantId, that.tenantId) &&
                Objects.equals(attribute1, that.attribute1) && Objects.equals(attribute2, that.attribute2) &&
                Objects.equals(attribute3, that.attribute3) && Objects.equals(attribute4, that.attribute4) &&
                Objects.equals(attribute5, that.attribute5);
    }

    @Override
    public int hashCode() {
        return Objects.hash(pistbId, ingotType, guaranteeQuality, minRollThick, maxRollThick, minRollWidth, maxRollWidth,
                rollLenFrom, rollLenTo, rollLineNo, withSideFlag, rollDirectCode, trimWidth, tenantId, attribute1, attribute2,
                attribute3, attribute4, attribute5);
    }
}
