package com.xisc.pm.domain.entity;

import com.fasterxml.jackson.annotation.JsonInclude;
import io.choerodon.mybatis.annotation.ModifyAudit;
import io.choerodon.mybatis.annotation.VersionAudit;
import io.choerodon.mybatis.domain.AuditDomain;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Objects;


/**
 * 组板临时收得率基准表(PmPdTmpYieldBt)实体类
 *
 * @author 纪文桃 wentao.ji01@hand-china.com
 * @since 2024-07-24 11:02:22
 */

@Getter
@Setter
@ApiModel("组板临时收得率基准表")
@VersionAudit
@ModifyAudit
@JsonInclude(value = JsonInclude.Include.NON_NULL)
@Table(name = "PM_PD_TMP_YIELD_BT")
public class PmPdTmpYieldBt extends AuditDomain implements Serializable {
    private static final long serialVersionUID = 871723493298552316L;

    public static final String FIELD_PPTYB_ID = "pptybId";
    public static final String FIELD_CC_TYPE = "ccType";
    public static final String FIELD_PROD_CODE_HP = "prodCodeHp";
    public static final String FIELD_WITH_SIDE_FLAG = "withSideFlag";
    public static final String FIELD_SLAB_THICK_FROM = "slabThickFrom";
    public static final String FIELD_SLAB_THICK_TO = "slabThickTo";
    public static final String FIELD_SLAB_WIDTH_FROM = "slabWidthFrom";
    public static final String FIELD_SLAB_WIDTH_TO = "slabWidthTo";
    public static final String FIELD_PLATE_THICK_FROM = "plateThickFrom";
    public static final String FIELD_PLATE_THICK_TO = "plateThickTo";
    public static final String FIELD_TMP_YIELD = "tmpYield";
    public static final String FIELD_ROLL_LINE_NO = "rollLineNo";
    public static final String FIELD_TENANT_ID = "tenantId";
    public static final String FIELD_ATTRIBUTE1 = "attribute1";
    public static final String FIELD_ATTRIBUTE2 = "attribute2";
    public static final String FIELD_ATTRIBUTE3 = "attribute3";
    public static final String FIELD_ATTRIBUTE4 = "attribute4";
    public static final String FIELD_ATTRIBUTE5 = "attribute5";

    @ApiModelProperty("主键ID")
    @Id
    @GeneratedValue
    private Long pptybId;

    @ApiModelProperty(value = "连铸机类型", required = true)
    @NotBlank
    private String ccType;

    @Transient
    @ApiModelProperty(value = "连铸机类型含义")
    private String ccTypeDesc;

    @ApiModelProperty(value = "厚板品种代码", required = true)
    @NotBlank
    private String prodCodeHp;

    @Transient
    @ApiModelProperty(value = "厚板品种代码含义")
    private String prodCodeHpDesc;

    @ApiModelProperty(value = "切边标记", required = true)
    @NotBlank
    private String withSideFlag;


    @Transient
    @ApiModelProperty(value = "切边标记含义")
    private String withSideFlagDesc;

    @ApiModelProperty(value = "板坯厚度起", required = true)
    @NotNull
    private BigDecimal slabThickFrom;

    @ApiModelProperty(value = "板坯厚度止", required = true)
    @NotNull
    private BigDecimal slabThickTo;

    @ApiModelProperty(value = "板坯宽度起", required = true)
    @NotNull
    private BigDecimal slabWidthFrom;

    @ApiModelProperty(value = "板坯宽度止", required = true)
    @NotNull
    private BigDecimal slabWidthTo;

    @ApiModelProperty(value = "钢板厚度起", required = true)
    @NotNull
    private BigDecimal plateThickFrom;

    @ApiModelProperty(value = "钢板厚度止", required = true)
    @NotNull
    private BigDecimal plateThickTo;


    @ApiModelProperty(value = "临时收得率", required = true)
    @NotNull
    private BigDecimal tmpYield;

    @ApiModelProperty(value = "轧制产线号", required = true)
    @NotBlank
    private String rollLineNo;

    @Transient
    @ApiModelProperty(value = "轧制产线号含义")
    private String rollLineNoDesc;

    @ApiModelProperty(value = "租户ID", required = true)
    @NotNull
    private Long tenantId;

    private String attribute1;

    private String attribute2;

    private String attribute3;

    private String attribute4;

    private String attribute5;


    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        PmPdTmpYieldBt that = (PmPdTmpYieldBt) o;
        return Objects.equals(ccType, that.ccType) && Objects.equals(ccTypeDesc, that.ccTypeDesc) &&
                Objects.equals(prodCodeHp, that.prodCodeHp) && Objects.equals(withSideFlag, that.withSideFlag) &&
                Objects.equals(withSideFlagDesc, that.withSideFlagDesc) && Objects.equals(slabThickFrom, that.slabThickFrom) &&
                Objects.equals(slabThickTo, that.slabThickTo) && Objects.equals(slabWidthFrom, that.slabWidthFrom) &&
                Objects.equals(slabWidthTo, that.slabWidthTo) && Objects.equals(plateThickFrom, that.plateThickFrom) &&
                Objects.equals(plateThickTo, that.plateThickTo) && Objects.equals(tmpYield, that.tmpYield) &&
                Objects.equals(rollLineNo, that.rollLineNo) && Objects.equals(rollLineNoDesc, that.rollLineNoDesc) &&
                Objects.equals(tenantId, that.tenantId);
    }

    @Override
    public int hashCode() {
        return Objects.hash(ccType, ccTypeDesc, prodCodeHp, withSideFlag, withSideFlagDesc, slabThickFrom, slabThickTo,
                slabWidthFrom, slabWidthTo, plateThickFrom, plateThickTo, tmpYield, rollLineNo, rollLineNoDesc, tenantId);
    }
}
